#include "stm32f7xx.h"
#include "adc_dma.h"

/*Pins
 * PA3 ->AIn_CH3
 * PC0 ->AIn_CH10
 * */

/*Required stream and channel
 *
 * Stream 0
 * Channel 0
 * */

uint16_t adc_data[2];

void adc_dma_2ch_init()
	{
	/*Enable clock access to GPIOA and GPIOC*/
	RCC->AHB1ENR|=RCC_AHB1ENR_GPIOAEN|RCC_AHB1ENR_GPIOCEN;
	/*Configure the pins in ADC mode*/
	GPIOA->MODER|=GPIO_MODER_MODER3;
	GPIOC->MODER|=GPIO_MODER_MODER0;

	/*Enable clock access to ADC1*/
	RCC->APB2ENR|=RCC_APB2ENR_ADC1EN;
	/*Enable scan mode*/
	ADC1->CR1|=ADC_CR1_SCAN;
	/*Enable continuous/ DMA and DDS*/
	ADC1->CR2|=ADC_CR2_CONT|ADC_CR2_DDS|ADC_CR2_DMA;
	/*Set length to 2 channels*/
	ADC1->SQR1|=(1<<20);
	/*Set ch3 first first and ch10 second in the conversion sequence*/
	ADC1->SQR3|=(3<<0)|(10<<5);
	/*Enable the ADC*/
	ADC1->CR2|=ADC_CR2_ADON;

	/*Enable clock access to DMA2*/
	RCC->AHB1ENR|=RCC_AHB1ENR_DMA2EN;
	/*Disable the strsam*/
	DMA2_Stream0->CR&=~DMA_SxCR_EN;
	/*Wait until the stream is disabled */
	while(DMA_SxCR_EN==(DMA2_Stream0->CR&DMA_SxCR_EN));
	/*Set memory size to half-word*/
	DMA2_Stream0->CR|=DMA_SxCR_MSIZE_0;
	/*Set peripheral size to half word*/
	DMA2_Stream0->CR|=DMA_SxCR_PSIZE_0;
	/*Enable memory increment*/
	DMA2_Stream0->CR|=DMA_SxCR_MINC;
	/*Enable circular mode*/
	DMA2_Stream0->CR|=DMA_SxCR_CIRC;
	/*Set peripheral is the controller*/
	DMA2_Stream0->CR|=DMA_SxCR_PFCTRL;
	/*Set number of transfer to 2*/
	DMA2_Stream0->NDTR=2;
	/*Set peripheral address to ADC1->DR*/
	DMA2_Stream0->PAR=(uint32_t)&(ADC1->DR);
	/*Set memory address to adc_data*/
	DMA2_Stream0->M0AR=(uint32_t)&(adc_data);
	/*Enable the DMA stream*/
	DMA2_Stream0->CR|=DMA_SxCR_EN;
	/*Start ADC conversion*/
	ADC1->CR2|=ADC_CR2_SWSTART;
	}

uint16_t ReadChannel(adc_Channel_t ch)
	{
	return adc_data[ch];
	}
